@@ -2,22 +2,22 @@
 //! 
 //! This crate provides convenient access to the binary file system of the [Oldschool RuneScape] and [RuneScape 3] caches.
 //!
-//! The library's API is mainly focussed around reading bytes easily.
-//! Therefore it offers a higher level of abstraction compared to other libraries. Most cache API's expose a
+//! The library's API is mainly focused around reading bytes easily.
+//! Therefore, it offers a higher level of abstraction compared to other libraries. Most cache API's expose a
 //! wide variety of internal types to let the user tinker around with the cache in unusual ways.
-//! To avoid undefined behavior most internal types are kept private.
+//! To avoid undefined behavior, most internal types are kept private.
 //! The goal of this crate is to provide a simple interface for basic reading of valuable data.
 //!
-//! Note that this crate is still evolving, both OSRS & RS3 are not fully supported/implemented and
+//! Note that this crate is still evolving; both OSRS & RS3 are not fully supported/implemented and
 //! will probably contain bugs or miss vital features. If this is the case for you then consider [opening
 //! an issue].
 //! 
 //! # Safety
 //! 
-//! This crate internally uses [memmap] and this is safe because: the RuneScape cache is a read-only binary files ystem 
-//! which is never modified by any process, and should never be modified. To ensure the main file is never moved while the
-//! cache has memory mapped to it a file handle is kept internally to make access more safe. It is not possible to prevent 
-//! parallel access to a certain file and prevent modifications. Therefore file-backed mapped memory is inherently unsafe.
+//! This crate internally uses [memmap] and this is safe because: the RuneScape cache is a read-only binary file system 
+//! which is never modified by any process, and should never be modified. [`Mmap`] provides basic file safety with
+//! [`std::fs::File`]. It is not possible to prevent parallel access to a certain file and prevent modifications. 
+//! Therefore file-backed mapped memory is inherently unsafe.
 //!
 //! # Features
 //!
@@ -56,6 +56,8 @@
 //! [opening an issue]: https://github.com/jimvdl/rs-cache/issues/new
 //! [serde]: https://crates.io/crates/serde
 //! [memmap]: https://crates.io/crates/memmap
+//! [`Mmap`]: https://docs.rs/memmap/0.7.0/memmap/struct.Mmap.html
+//! [`std::fs::File`]: https://doc.rust-lang.org/std/fs/struct.File.html
 
 #![deny(clippy::all, clippy::nursery)]
 #![warn(
@@ -94,7 +96,10 @@
 
 // TODO: add rust-version to [package]
 // TODO: run cargo outdated when it is fixed for stable 1.56
-
+// TODO: maybe refactor read internal to include ::std::cmp::min() -> min(buf.len(), state.data.len()) 
+// buf[..n].copy_from_slice(data[..n]) see 1:13:41
+// TODO: reorder all structs and impl blocks in crate
+ 
 #[macro_use]
 pub mod util;
 mod archive;
@@ -127,13 +132,12 @@ use crate::{
     checksum::{Checksum, Entry},
     error::{ParseError, ReadError},
     index::Indices,
-    sector::{Sector, SectorHeaderSize, SECTOR_SIZE},
+    sector::{Sector, SECTOR_SIZE},
 };
 
 /// A parsed Jagex cache.
 #[derive(Debug)]
 pub struct Cache {
-    main_file: File,
     data: Mmap,
     indices: Indices,
 }
@@ -157,7 +161,7 @@ impl Cache {
         let data = unsafe { Mmap::map(&main_file)? };
         let indices = Indices::new(path, &data)?;
 
-        Ok(Self { main_file, data, indices })
+        Ok(Self { data, indices })
     }
 
     /// Reads from the internal data.
@@ -186,6 +190,8 @@ impl Cache {
         let mut buffer = Vec::with_capacity(archive.length);
         self.data.read_internal(archive, &mut buffer)?;
 
+        assert_eq!(buffer.len(), archive.length);
+
         Ok(buffer)
     }
 
@@ -315,49 +321,23 @@ pub(crate) trait ReadInternal {
 impl ReadInternal for Mmap {
     #[inline]
     fn read_internal<W: Write>(&self, archive: &ArchiveRef, writer: &mut W) -> crate::Result<()> {
-        let header_size = SectorHeaderSize::from_archive(archive);
-        let (header_len, data_len) = header_size.clone().into();
         let mut current_sector = archive.sector;
-        let mut remaining = archive.length;
-        let mut chunk = 0;
+        let (header_size, chunks) = archive.chunks();
 
-        loop {
-            let offset = current_sector as usize * SECTOR_SIZE;
-            if remaining >= data_len {
-                let data_block = &self[offset..offset + SECTOR_SIZE];
-                match Sector::new(data_block, &header_size) {
-                    Ok(sector) => {
-                        sector
-                            .header
-                            .validate(archive.id, chunk, archive.index_id)?;
-                        current_sector = sector.header.next;
-                        writer.write_all(sector.data_block)?;
-                    }
-                    Err(_) => return Err(ParseError::Sector(archive.sector).into()),
-                };
+        for (chunk, data_len) in chunks.enumerate() {
+            let offset = current_sector * SECTOR_SIZE;
 
-                remaining -= data_len;
-            } else {
-                if remaining == 0 {
-                    break;
+            let data_block = &self[offset..offset + data_len];
+            match Sector::new(data_block, &header_size) {
+                Ok(sector) => {
+                    sector
+                        .header
+                        .validate(archive.id, chunk, archive.index_id)?;
+                    current_sector = sector.header.next;
+                    writer.write_all(sector.data_block)?;
                 }
-
-                let data_block = &self[offset..offset + remaining + header_len];
-
-                match Sector::new(data_block, &header_size) {
-                    Ok(sector) => {
-                        sector
-                            .header
-                            .validate(archive.id, chunk, archive.index_id)?;
-                        writer.write_all(sector.data_block)?;
-
-                        break;
-                    }
-                    Err(_) => return Err(ParseError::Sector(archive.sector).into()),
-                };
-            }
-
-            chunk += 1;
+                Err(_) => return Err(ParseError::Sector(archive.sector).into()),
+            };
         }
 
         Ok(())
